# Polythene CSS

<!-- MarkdownTOC autolink="true" autoanchor="true" bracket="round" levels="1,2,3" -->

- [Basic principles](#basic-principles)
  - [Getting started](#getting-started)
  - [Using CSS-in-JS](#using-css-in-js)
    - [Usage](#usage)
    - [Theming options](#theming-options)
    - [Under the hood](#under-the-hood)
  - [Using CSS files](#using-css-files)
    - [Usage](#usage-1)
    - [Theming](#theming)
    - [Alternative approach](#alternative-approach)
- [Extras](#extras)
  - [Supporting styles](#supporting-styles)
    - [Using CSS-in-JS](#using-css-in-js-1)
    - [Using CSS files](#using-css-files-1)
  - [Roboto font](#roboto-font)
    - [Roboto using CSS-in-JS](#roboto-using-css-in-js)
    - [Roboto using CSS files](#roboto-using-css-files)
  - [Layout classes](#layout-classes)
      - [Mithril example](#mithril-example)
      - [React JSX example](#react-jsx-example)
    - [Layout classes using CSS-in-JS](#layout-classes-using-css-in-js)
    - [Layout classes using CSS files](#layout-classes-using-css-files)
    - [List of layout classes](#list-of-layout-classes)

<!-- /MarkdownTOC -->


<a id="basic-principles"></a>
## Basic principles

Polythene styles can be included in 2 ways:

1. Using CSS-in-JS
1. Importing/linking CSS files


Which option you choose may depend on how (and if) you want to use theming - more on that below.


<a id="getting-started"></a>
### Getting started

With either option, `polythene-css` should be installed - see [Getting started with Mithril](getting-started-mithril.md) or [Getting started with React](getting-started-react.md).



<a id="using-css-in-js"></a>
### Using CSS-in-JS

Using CSS-in-JS has minimal setup costs and gives the possibility to add themed component styles on the fly.


#### Usage

Importing `polythene-css` activates CSS-in-JS:

~~~javascript
import "polythene-css"
~~~


#### Theming options

Because styles are generated by JavaScript, no CSS-build step is required. Adding a component theme is as simple as: `addStyle(".themed-component", { vars })`. For example:

~~~javascript
IconCSS.addStyle(".my-icon", {
  size_regular: 44,
  color_light:  "purple"
})
~~~

Read more at [Theming Style variables](theming/style-variables.md).


#### Under the hood

Component styles are automatically added to `<head>` using [j2c](http://j2c.py.gy).



<a id="using-css-files"></a>
### Using CSS files

Choose this:
* If you don't need to create themed component styles on the fly
* or if you are already using a bundler that grabs and bundles all CSS files.
* or if you don't want to be reliant on the client rendering, for example when doing server-side rendering or when using a static site generator.


#### Usage

Package `polythene-css` contains all component CSS files. Its existence in `node_modules` makes it possible to import CSS files.

Most bundlers have options for importing CSS files. Frequently used with Webpack is [mini-css-extract-plugin](https://github.com/webpack-contrib/mini-css-extract-plugin).

Importing all CSS:

~~~javascript
import "polythene-css/dist/polythene.css"            // Component CSS
import "polythene-css/dist/polythene-typography.css" // Default Material Design styles including Roboto font
~~~

Alternatively, import a single component's CSS:

~~~javascript
import "polythene-css/dist/polythene-button.css"
~~~


#### Theming

Theming options are more limited than using CSS-in-JS as styles will not be added on the fly. But it is still possible to use the same tools as CSS-in-JS, only with an extra build step that results in a CSS file. This step can be automated of course.

[polythene-scripts](packages/polythene-scripts.md) provides the tooling for writing CSS based on component variables.

For example, to create a CSS file that contains themed CSS for Card and Button, you could write:

~~~javascript
// ./scripts/writeThemeCSS.js
const { writeCSS } = require("polythene-scripts")
const { CardCSS, ButtonCSS } = require("polythene-css")

const styles = [
  CardCSS.getStyle(".themed-card", {
    color_dark_main_background: "#B89E58",
    color_dark_title_text:      "#fff",
    color_dark_subtitle_text:   "#fff"
  }),
  ButtonCSS.getStyle(".themed-button", {
    color_light_text: "red"
  })
]

writeCSS({
  styles,
  path: "./dist/css/theme.css",
  autoPrefix: true,
  gzip: true,
})
~~~

... and in your `package.json` build script:

~~~json
"build": "npm run writeCSS && webpack --config scripts/webpack.config.prod.js"
~~~



#### Alternative approach

You could always resort to create handwritten CSS. Although this does not require a build step, this may not be future proof. When a component structure would change later on, your custom CSS may no longer work.



<a id="extras"></a>
## Extras


<a id="supporting-styles"></a>
### Supporting styles

Supporting styles (typography and Roboto web font styles) are optional.


#### Using CSS-in-JS

~~~javascript
import { addTypography } from "polythene-css"

addTypography()
~~~

See also:

* [Loading web fonts](packages/polythene-utilities.md#web-font-loader)
* [Writing custom CSS](theming/custom-css.md)


#### Using CSS files

~~~
import "polythene-css/dist/polythene-typography.css"
~~~


<a id="roboto-font"></a>
### Roboto font


#### Roboto using CSS-in-JS

The Material Design Roboto font is automatically loaded when `addTypography` is used, but it can also be loaded independent of typography styles:

~~~javascript
import { addRoboto } from "polythene-css"

addRoboto()
~~~

The Material Design Roboto font is loaded from Google Fonts using [webfontloader](https://github.com/typekit/webfontloader).
The benefit of using this approach is that when the font is loaded, styles are added to the `html` tag, giving simple hooks to change styles based on the loaded state.

For example, to prevent the [Flash of Unstyled Text (FOUT)](https://www.paulirish.com/2009/fighting-the-font-face-fout/), add these styles:

~~~css
body {
  opacity: 0
}
html.wf-active body {
  opacity: 1
}
~~~

Or with CSS-in-JS using `styler`:

~~~javascript
import { styler } from "polythene-core-css"

const foutStyles = [{
  "body": {
    opacity: 0
  },
  "html.wf-active body": {
    opacity: 1
  }
}]

styler.add("fout", foutStyles)
~~~


#### Roboto using CSS files

Roboto will be loaded automatically when `polythene-typography` is imported:

~~~
import "polythene-css/dist/polythene-typography.css"
~~~




<a id="layout-classes"></a>
### Layout classes

Package `polythene-css` also includes helper classes that are useful to quickly add layout styles to hyperscript / JSX (see the complete list of layout classes below). 

##### Mithril example

~~~javascript
m(".layout.vertical", "Vertical content")
~~~

##### React JSX example

~~~javascript
<div className="layout vertical">Vertical content</div>
~~~


#### Layout classes using CSS-in-JS

~~~javascript
import { addLayoutStyles } from "polythene-css"

addLayoutStyles()
~~~

#### Layout classes using CSS files

~~~javascript
import "polythene-css/dist/polythene-layout-styles.css"
~~~



#### List of layout classes 

Common helper classes:

~~~css
.pe-block
.pe-inline-block
.pe-hidden
.pe-relative
.pe-absolute
.pe-fit
.pe-fullbleed
~~~

Flex classes:

~~~css
/* flex */
.flex
.flex.auto
.flex.auto-vertical
.flex.none
.flex.one
.flex.two
.flex.three
.flex.four
.flex.five
.flex.six
.flex.seven
.flex.eight
.flex.nine
.flex.ten
.flex.eleven
.flex.twelve

/* layout */
.layout
.layout.horizontal
.layout.horizontal.inline
.layout.vertical.inline
.layout.horizontal
.layout.horizontal.reverse
.layout.vertical
.layout.vertical.reverse
.layout.wrap
.layout.wrap.reverse

/* alignment in cross axis */
.layout.start
.layout.center,
.layout.center-center
.layout.end

/* alignment in main axis */
.layout.start-justified
.layout.center-justified
.layout.center-center
.layout.end-justified
.layout.around-justified
.layout.justified

/* self alignment */
.self-start
.self-center
.self-end
.self-stretch
~~~
